﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_ownerdraw_listview
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Make some data.
        private void Form1_Load(object sender, EventArgs e)
        {
            ListView[] listViews = new ListView[] { lvwList, lvwSmallIcon, lvwLargeIcon, lvwTile, lvwDetails };
            foreach (ListView lvw in listViews)
            {
                AddItem(lvw, "Butterfly", Properties.Resources.Butterfly, Color.Green);
                AddItem(lvw, "Guppy", Properties.Resources.Fish, Color.Red);
                AddItem(lvw, "Peggy", Properties.Resources.Peggy, Color.Yellow);
            }
        }

        // Make a server status item.
        private void AddItem(ListView lvw, string server, Image logo, Color status)
        {
            // Make the item.
            ListViewItem item = new ListViewItem(server);

            // Save the ServeStatus item in the Tag property.
            ServerStatus server_status = new ServerStatus(server, logo, status);
            item.Tag = server_status;
            item.SubItems[0].Name = "Server";

            // Add subitems so they can draw.
            item.SubItems.Add("Logo");
            item.SubItems.Add("Status");

            // Add the item to the ListView.
            lvw.Items.Add(item);
        }

        // Just draw the column's text.
        private void lvwServers_DrawColumnHeader(object sender, DrawListViewColumnHeaderEventArgs e)
        {
            using (StringFormat string_format = new StringFormat())
            {
                string_format.Alignment = StringAlignment.Center;
                string_format.LineAlignment = StringAlignment.Center;

                string text = lvwList.Columns[e.ColumnIndex].Text;
                switch (e.ColumnIndex)
                {
                    case 0:
                        e.Graphics.DrawString(text, lvwList.Font, Brushes.Black, e.Bounds);
                        break;
                    case 1:
                        e.Graphics.DrawString(text, lvwList.Font, Brushes.Blue, e.Bounds);
                        break;
                    case 2:
                        e.Graphics.DrawString(text, lvwList.Font, Brushes.Green, e.Bounds);
                        break;
                }
            }
        }

        // Draw the item. In this case, the server's logo.
        private void lvwServers_DrawItem(object sender, DrawListViewItemEventArgs e)
        {
            // Draw Details view items in the DrawSubItem event handler.
            ListView lvw = e.Item.ListView;
            if (lvw.View == View.Details) return;

            // Get the ListView item and the ServerStatus object.
            ListViewItem item = e.Item;
            ServerStatus server_status = item.Tag as ServerStatus;

            // Clear.
            e.DrawBackground();

            // Draw a status indicator.
            e.Graphics.SmoothingMode =
                System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            Rectangle rect = new Rectangle(
                e.Bounds.Left + 1, e.Bounds.Top + 1,
                e.Bounds.Height - 2, e.Bounds.Height - 2);
            using (SolidBrush br = new SolidBrush(server_status.StatusColor))
            {
                e.Graphics.FillEllipse(br, rect);
            }
            e.Graphics.DrawEllipse(Pens.Black, rect);
            int left = rect.Right + 2;

            // See how much we must scale it.
            float scale;
            scale = e.Bounds.Height / (float)server_status.Logo.Height;

            // Scale and position the image.
            e.Graphics.ScaleTransform(scale, scale);
            e.Graphics.TranslateTransform(
                left,
                e.Bounds.Top + (e.Bounds.Height - server_status.Logo.Height * scale) / 2,
                System.Drawing.Drawing2D.MatrixOrder.Append);

            // Draw the image.
            e.Graphics.DrawImage(server_status.Logo, 0, 0);

            // Draw the focus rectangle if appropriate.
            e.Graphics.ResetTransform();
            e.DrawFocusRectangle();
        }

        // Draw subitems for Detail view.
        private void lvwServers_DrawSubItem(object sender, DrawListViewSubItemEventArgs e)
        {
            // Get the ListView item and the ServerStatus object.
            ListViewItem item = e.Item;
            ServerStatus server_status = item.Tag as ServerStatus;

            // Draw.
            switch (e.ColumnIndex)
            {
                case 0:
                    // Draw the server's name.
                    e.Graphics.DrawString(server_status.ServerName,
                        lvwList.Font, Brushes.Black, e.Bounds);
                    break;
                case 1:
                    // Draw the server's logo.
                    float scale = e.Bounds.Height / (float)server_status.Logo.Height;
                    e.Graphics.ScaleTransform(scale, scale);
                    e.Graphics.TranslateTransform(
                        e.Bounds.Left,
                        e.Bounds.Top + (e.Bounds.Height - server_status.Logo.Height * scale) / 2,
                        System.Drawing.Drawing2D.MatrixOrder.Append);
                    e.Graphics.DrawImage(server_status.Logo, 0, 0);
                    break;
                case 2:
                    // Draw the server's status.
                    Rectangle rect = new Rectangle(
                        e.Bounds.Left + 1, e.Bounds.Top + 1,
                        e.Bounds.Width - 2, e.Bounds.Height - 2);
                    using (SolidBrush br = new SolidBrush(server_status.StatusColor))
                    {
                        e.Graphics.FillRectangle(br, rect);
                    }
                    Color pen_color = Color.FromArgb(255,
                        255 - server_status.StatusColor.R,
                        255 - server_status.StatusColor.G,
                        255 - server_status.StatusColor.B);
                    using (SolidBrush br = new SolidBrush(pen_color))
                    {
                        using (StringFormat string_format = new StringFormat())
                        {
                            string_format.Alignment = StringAlignment.Center;
                            string_format.LineAlignment = StringAlignment.Center;
                            using (Font font = new Font(lvwList.Font, FontStyle.Bold))
                            {
                                e.Graphics.DrawString(server_status.StatusColor.Name,
                                    font, br, e.Bounds, string_format);
                            }
                        }
                    }
                    break;
            }

            // Draw the focus rectangle if appropriate.
            e.Graphics.ResetTransform();
            ListView lvw = e.Item.ListView;
            if (lvw.FullRowSelect)
            {
                e.DrawFocusRectangle(e.Item.Bounds);
            }
            else if (e.SubItem.Name == "Server")
            {
                e.DrawFocusRectangle(e.Bounds);
            }
        }
    }
}
